package com.hyp.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

@Slf4j
public class HypFileUtil extends FileUtil {

    public static final String  IE_FLAG_MISE = "msie";
    public static final String  IE_FLAG_TRIDENT = "trident";
    public static final String  IE_FLAG_EDGE = "edge";

    /**
     * MultipartFile转File
     */
    public static File toFile(MultipartFile multipartFile) {
        // 获取文件名
        String fileName = multipartFile.getOriginalFilename();
        // 获取文件后缀
        String prefix = "." + getExtensionName(fileName);
        File file = null;
        try {
            // 用uuid作为文件名，防止生成的临时文件重复
            file = File.createTempFile(IdUtil.simpleUUID(), prefix);
            // MultipartFile to File
            multipartFile.transferTo(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return file;
    }

    /**
     * 获取文件扩展名，不带.
     */
    public static String getExtensionName(String filename) {
        if ((filename != null) && (filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length() - 1))) {
                return filename.substring(dot + 1);
            }
        }
        return filename;
    }



    /**
     * 获取不带扩展名的文件名
     */
    public static String getFileNameNoEx(String filename) {
        if ((filename != null) && (filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length()))) {
                return filename.substring(0, dot);
            }
        }
        return filename;
    }


    public static File upload(MultipartFile file, String uploadDir, String applyId){
        String suffix = getExtensionName(file.getOriginalFilename());
        String fileId = RandomUtil.randomNumbers(32);
        String dateDir = DateUtil.format(DateUtil.date(),"yyyyMMdd");
        //上传文件以后的存储路径
        String separator = File.separator;
        String filePath = uploadDir +  separator + dateDir + separator + applyId + separator + fileId + "." + suffix;
        try {
            File upFile = new File(filePath).getCanonicalFile();
            // 检测是否存在目录
            if (!upFile.getParentFile().exists()) {
                upFile.getParentFile().mkdirs();
            }
            log.info("上传文件路径：{}", filePath);
            // 文件写入
            file.transferTo(upFile);
            return upFile;
        } catch (IOException e) {
            throw new RuntimeException("上传失败");
        }
    }


    public static void downLoadFile(String filePath, String fileName, HttpServletRequest request, HttpServletResponse response) {

        File f = null;
        try {
            f = FileUtil.newFile(filePath);
            if (!f.exists()) {
                response.sendError(404, "File not found!");
                return;
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
        try (BufferedInputStream br = new BufferedInputStream(new FileInputStream(f));
             OutputStream out = response.getOutputStream();) {
            byte[] buf = new byte[1024];
            int len = 0;
            // 在线打开方式
            String userAgent = request.getHeader("User-Agent").toLowerCase();
            fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
            response.reset();
            // 解决申报系统异步下载跨域问题
            response.setHeader("Access-Control-Allow-Origin", "*");
            response.setHeader("Access-Control-Allow-Methods", "POST, GET");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            // 纯下载方式
            response.setHeader("Content-disposition", String.format("attachment; filename=\"%s\"", fileName));
            response.setContentType("application/x-msdownload;charset=UTF-8");
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            while ((len = br.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

}
